<?php

namespace HadonaCloaker\Controller;

class HccDatabase
{
  protected $wpdb, $info, $cloaker;

  public function __construct()
  {
    global $wpdb;
    $this->wpdb = $wpdb;
    $this->cloaker = $wpdb->prefix . 'hcc_cloakers';
    $this->info = $wpdb->prefix . 'hcc_user_info';
  }

  public function create_tables() {
    $charset_collate = $this->wpdb->get_charset_collate();

    $cloaker_sql = "CREATE TABLE $this->cloaker (
      id INT(9) NOT NULL AUTO_INCREMENT,
      cloaker_name VARCHAR(255) NOT NULL,
      cloaker_codition VARCHAR(255) NOT NULL,
      cloaker_region VARCHAR(255),
      cloaker_method VARCHAR(255),
      default_id INT(9),
      default_title VARCHAR(255),
      cloaker_id INT(9),
      cloaker_title VARCHAR(255),
      redirect_domain VARCHAR(255),
      status ENUM('deactivated','activated') DEFAULT 'activated',
      created_at TIMESTAMP NOT NULL,
      updated_at TIMESTAMP NOT NULL,
      PRIMARY KEY (id)
    ) $charset_collate;";

    $info_sql = "CREATE TABLE $this->info (
      id INT(9) NOT NULL AUTO_INCREMENT,
      token VARCHAR(255) NOT NULL,
      expired_date DATE NOT NULL,
      activation_status VARCHAR(255) NOT NULL,
      PRIMARY KEY (id)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

    dbDelta($cloaker_sql);
    dbDelta($info_sql);

    if (get_site_option('hcc_ver')) {
			update_option('hcc_ver', HCC_VER);
		} else {
			add_option('hcc_ver', HCC_VER);
		}
  }

  /**
   * Cloaker Database Actions
   */

  public function store_cloaker($request)
  {
    return $this->wpdb->insert(
      $this->cloaker,
      [
        'cloaker_name' => $request['cloaker_name'],
        'cloaker_codition' => $request['cloaker_codition'],
        'cloaker_region' => $request['cloaker_region'],
        'cloaker_method' => $request['cloaker_method'],
        'default_id' => $request['default_id'],
        'default_title' => $request['default_title'],
        'cloaker_id' => $request['cloaker_id'],
        'cloaker_title' => $request['cloaker_title'],
        'redirect_domain' => $request['redirect_domain'],
        'status' => 'activated',
      ],
      ['%s', '%s', '%s', '%s', '%d', '%s', '%d', '%s', '%s'],
    );
  }

  public function get_cloakers($offset = 0, $per_page = 10, $keyword = "%%")
  {
    $list_cloakers['data'] = $this->wpdb->get_results(
      $this->wpdb->prepare(
        "SELECT * FROM $this->cloaker
        WHERE cloaker_name LIKE %s
        ORDER BY id DESC
        LIMIT %d,%d",
        $keyword,
        $offset,
        $per_page
      )
    );

    $list_cloakers['total_data'] = $this->wpdb->get_var(
      $this->wpdb->prepare("SELECT COUNT(*) FROM $this->cloaker WHERE cloaker_name LIKE %s", $keyword)
    );

    return $list_cloakers;
  }

  public function get_cloaker_data($cloaker_id)
  {
    $cloaker = $this->wpdb->get_row(
      $this->wpdb->prepare(
        "SELECT * FROM $this->cloaker
        WHERE id = %d",
        $cloaker_id,
      )
    );

    return $cloaker;
  }

  public function update_cloaker($request, $cloaker_id)
  {
    $cloaker_updated = $this->wpdb->update(
      $this->cloaker,
      [
        'cloaker_name' => $request['cloaker_name'],
        'cloaker_codition' => $request['cloaker_codition'],
        'cloaker_region' => $request['cloaker_region'],
        'cloaker_method' => $request['cloaker_method'],
        'default_id' => $request['default_id'],
        'default_title' => $request['default_title'],
        'cloaker_id' => $request['cloaker_id'],
        'cloaker_title' => $request['cloaker_title'],
        'redirect_domain' => $request['redirect_domain'],
      ],
      ['id' => $cloaker_id],
      ['%s', '%s', '%s', '%s', '%d', '%s', '%d', '%s', '%s'],
      ['%d']
    );

    return $cloaker_updated;
  }

  public function update_cloaker_status($status, $cloaker_id)
  {
    $cloaker_updated_status = $this->wpdb->update(
      $this->cloaker,
      [
        'status' => $status,
      ],
      ['id' => $cloaker_id],
      ['%s'],
      ['%d']
    );

    return $cloaker_updated_status;
  }

  public function delete_cloaker($cloaker_id)
  {
    if ( !empty( $cloaker_id ) ) {
      return $this->wpdb->delete(
        $this->cloaker,
        ['id' => $cloaker_id]
      );
    } else {
      return false;
    }
  }

  /**
   * User Info Database Actions
   */
  public function store_info($request)
  {
    return $this->wpdb->insert(
      $this->info,
      [
        'token' => $request['token'],
        'expired_date' => $request['expired_date'],
        'activation_status' => $request['activation_status']
      ],
      ['%s', '%s', '%s']
    );
  }

  public function update_info($request, $user_id)
  {
    $info_updated = $this->wpdb->update(
      $this->info,
      [
        'expired_date' => $request['expired_date'],
        'activation_status' => $request['activation_status']
      ],
      ['id' => $user_id],
      ['%s', '%s'],
      ['%d']
    );

    return $info_updated;
  }

  public function get_user_info()
  {
    $user_info = $this->wpdb->get_row(
      "SELECT * FROM $this->info"
    );

    return $user_info;
  }

  public function delete_user_info($user_id)
  {
    return $this->wpdb->delete(
      $this->info,
      ['id' => $user_id]
    );
  }
}