<?php

namespace HadonaCloaker\Controller;

use WP_REST_Request;
use WP_REST_Controller;

use HadonaCloaker\Controller\HccDatabase;

class HccRestApi extends WP_REST_Controller
{
  public function init()
  {
    add_action('wp_ajax_show_cloakers', [$this, 'show_cloakers']);
    add_action('wp_ajax_get_cloaker_info', [$this, 'get_cloaker_info']);
    add_action('wp_ajax_create_cloaker', [$this, 'create_cloaker']);
    add_action('wp_ajax_update_cloaker', [$this, 'update_cloaker']);
    add_action('wp_ajax_update_cloaker_status', [$this, 'update_cloaker_status']);
    add_action('wp_ajax_delete_cloaker', [$this, 'delete_cloaker']);
    add_action('wp_ajax_store_user_info', [$this, 'store_user_info']);
    add_action('wp_ajax_update_user_info', [$this, 'update_user_info']);
    add_action('wp_ajax_delete_user_info', [$this, 'delete_user_info']);
  }

  private function json_responses($result) {
		if($result) {
			wp_send_json_success(['message' => 'success'], 200);
		} else {
			wp_send_json_error(['message' => 'failed'], 400);
		}
	}

  public function show_cloakers()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();

    $per_page = isset($_POST['per_page']) ? $_POST['per_page'] : 10;
    $page = isset($_POST['page']) ? $_POST['page'] : 1;
    $offset = ($page - 1) * $per_page;

    $keyword = isset($_POST['search']) && !empty($_POST['search']) ? "%". $_POST['search'] . "%" : '%%';

    $cloakers = $db_class->get_cloakers($offset, $per_page, $keyword);
    $total_page = ceil($cloakers['total_data'] / $per_page);

    wp_send_json([
      'data' => $cloakers['data'],
      'total_data' => $cloakers['total_data'],
      'total_page' => $total_page,
      'has_next' => $page < $total_page,
      'has_prev' => $page > 1
    ]);
  }

  public function create_cloaker() {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();

    $cloaker_created = $db_class->store_cloaker($_POST['payload']);
    $this->json_responses($cloaker_created);
  }

  public function get_cloaker_info()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();
    $cloaker_data = $db_class->get_cloaker_data($_POST['id']);
		wp_send_json(['data' => $cloaker_data]);
  }

  public function update_cloaker()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();
    $cloaker_updated = $db_class->update_cloaker($_POST['payload'], $_POST['id']);

    $this->json_responses($cloaker_updated);
  }

  public function update_cloaker_status()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();
    $cloaker_updated = $db_class->update_cloaker_status($_POST['status'], $_POST['id']);

    $this->json_responses($cloaker_updated);
  }

  public function delete_cloaker()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();
    $cloaker_deleted = $db_class->delete_cloaker($_POST['id']);
    $this->json_responses($cloaker_deleted);
  }

  public function store_user_info()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();

    $cloaker_created = $db_class->store_info($_POST['payload']);
    $this->json_responses($cloaker_created);
  }

  public function update_user_info()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();

    $user_updated = $db_class->update_info($_POST['payload'], $_POST['id']);
    $this->json_responses($user_updated);
  }

  public function delete_user_info()
  {
    check_ajax_referer('hadona-cloaker', 'security');

    $db_class = new HccDatabase();
    $user_deleted = $db_class->delete_user_info($_POST['id']);
    $this->json_responses($user_deleted);
  }
}

