<?php

namespace HadonaCloaker\Controller;

use HadonaCloaker\Controller\HccDatabase;

class HccShortcode
{
  public function init()
  {
    add_shortcode('hadona-cloaker', [$this, 'cloaker_redirect'], 1);
  }

  public function cloaker_redirect($atts)
  {
    $is_valid = $this->check_plugin_status();

    if ($is_valid) {
      if (isset($atts['id'])) {
        $cloaker = $this->get_cloaker_data($atts['id']);

        if ($cloaker->status == 'activated') {
          return $this->cloak($cloaker->cloaker_method, $cloaker->cloaker_codition, $cloaker->default_id, $cloaker->cloaker_id, $cloaker->redirect_domain, $cloaker->cloaker_region);
        }
      } elseif (isset($atts['default']) && isset($atts['cloaker'])) {
        $default_page_slug = $atts['default'];
        $replacement_page_slug = $atts['cloaker'];
        $redirect_method = isset($atts['method']) ? $atts['method'] : 'robot';

        if (is_numeric($atts['default']) && is_numeric($atts['cloaker'])) {
          return $this->cloak('replace', $redirect_method, $default_page_slug, $replacement_page_slug, '', 'all');
        } else {
          return 'Please insert page id';
        }
      }
    }
  }

  private function check_user_country()
  {
    $user_ip = $_SERVER['REMOTE_ADDR'];
    $location = file_get_contents("http://www.geoplugin.net/php.gp?ip='{$user_ip}");
    $location_data = unserialize($location);

    return $location_data['geoplugin_countryCode'];
  }

  private function check_plugin_status()
  {
    $db_class = new HccDatabase();
    $user = $db_class->get_user_info();

    $is_active = $user->activation_status != 'blocked';

    date_default_timezone_set('Asia/Jakarta');
    $current_time = date('Y-m-d');
    $is_expired = $user->expired_date < $current_time;

    return $is_active && !$is_expired;
  }

  private function get_cloaker_data($cloaker_id)
  {
    $db_class = new HccDatabase();
    $cloaker = $db_class->get_cloaker_data($cloaker_id);

    return $cloaker;
  }

  private function replace_content($method, $condition, $default_page_content, $replacement_page_content, $url, $region)
  {
    $has_expected_query = isset($_GET['gclid']) || isset($_GET['fbclid']) || isset($_GET['ttclid']) || isset($_GET['CampaignID']);
    
    $user_region = $this->check_user_country();
    $is_on_region = false;

    if ($region !== 'all') {
      $selected_region = explode(',', $region);
      $is_on_region = in_array($user_region, $selected_region);
    }

    if ($condition && $has_expected_query && ($region == 'all' || $is_on_region)) {
      if ($method == 'replace') {
        return $this->elementor_replace_content($default_page_content);   
      } else {
        $this->url_redirect($url);
      }
    } else {
      if ($method == 'replace') {
        return $this->elementor_replace_content($replacement_page_content);
      }
    }
  }

  private function url_redirect($url)
  {
    $actual_link = "https://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

    wp_redirect( $url, 301 );  
    if ($actual_link !== $url) {
      echo("<script>window.location.href = '{$url}'</script>");	
    }
  }

  private function elementor_replace_content($post_id)
  {
    wp_enqueue_style('cloaker-replace-css', get_site_url() . '/wp-content/uploads/elementor/css/post-'. $post_id .'.css', array());
    
    $api_url = get_site_url() . '/wp-json/wp/v2/pages/' . $post_id;
    $response = wp_remote_get($api_url);

    if (!is_wp_error($response)) {
      $body = wp_remote_retrieve_body($response);
      $data = json_decode($body);
      return $data->content->rendered . "<br>";
    }
  }

  private function is_bot()
  {
    return preg_match('/bot|crawl|curl|dataprovider|search|get|spider|find|java|majesticsEO|google|yahoo|teoma|contaxe|yandex|libwww-perl|facebookexternalhit/i', $_SERVER['HTTP_USER_AGENT']);
  }

  private function cloak($cloaker_method, $method, $default_page_slug, $replacement_page_slug, $url, $region)
  {
    if (is_page() || is_single() && !$_GET['action']='elementor') {
      $default_page = $default_page_slug;
      $replacement_page = $replacement_page_slug;

      if ($method == 'robot' || $method == 'Robot') {
        return $this->replace_content($cloaker_method, $this->is_bot(), $default_page, $replacement_page, $url, $region);
      }

      if ($method == 'mobile' || $method == 'Mobile') {
        return $this->replace_content($cloaker_method, wp_is_mobile(), $default_page, $replacement_page, $url, $region);
      }

      if ($method == 'desktop' || $method == 'Desktop') {
        return $this->replace_content($cloaker_method, !wp_is_mobile(), $default_page, $replacement_page, $url, $region);
      }

      if ($method == 'all_devices') {
        $all_devices = !wp_is_mobile() || wp_is_mobile();

        return $this->replace_content($cloaker_method, $all_devices, $default_page, $replacement_page, $url, $region);
      }

      if ($method == 'all_methods') {
        $all_methods = !wp_is_mobile() || wp_is_mobile() || $this->is_bot();

        return $this->replace_content($cloaker_method, $all_methods, $default_page, $replacement_page, $url, $region);
      }
    }
  }
}